/* VzImagination (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
 * $Id: TestMatManipGaussianBlur.cpp 2741 2019-04-09 17:04:20Z stephane $
 */

#include "TestUtil.hpp"


TEST(GaussianBlur, Invalid)
{
	test_images();

	cv::Mat src;
	ASSERT_THROW(vz::apply_gaussian_blur(src), std::exception);

	src = cv::imread("capture.jpg");
	ASSERT_THROW(vz::apply_gaussian_blur(src, -1), std::exception);
	ASSERT_THROW(vz::apply_gaussian_blur(src,  0), std::exception);
	ASSERT_THROW(vz::apply_gaussian_blur(src,  2), std::exception);
}


TEST(GaussianBlur, DefaultKernelIs3x3)
{
	for (const auto & [path, name] : test_images())
	{
		std::cout << name << std::endl;

		cv::Mat src = cv::imread(path);
		ASSERT_EQ(src.channels(), 3);

		cv::Mat dst1 = vz::apply_gaussian_blur(src);
		cv::Mat dst2 = vz::apply_gaussian_blur(src, 3);
		ASSERT_EQ(src.channels(), 3); // source should not have changed
		ASSERT_EQ(dst1.channels(), 3);
		ASSERT_EQ(dst2.channels(), 3);
		ASSERT_TRUE(vz::compare_exact(dst1, dst2));
	}
}


TEST(GaussianBlur, DifferentKernels)
{
	// Try a Gaussian blur with different kernel sizes.  All images should be different,
	// getting progressively blurrier as the kernel size is increased.

	for (const auto & [path, name] : complex_test_images())
	{
		cv::Mat src = cv::imread(path);
		ASSERT_EQ(src.channels(), 3);
		cv::Mat copy = src.clone();

		vz::VMats v;
		for (int k : { 3, 5, 7, 9, 11, 13, 15, 25, 55, 95 })
		{
			std::cout << name << ": applying a Gaussian blur with a kernel size of " << k << std::endl;
			cv::Mat dst = vz::apply_blur_and_convert_to_greyscale(src, k);
			ASSERT_FALSE(vz::compare_exact(src, dst));

			// compare against all the other images we've blurred so far
			for (cv::Mat rhs : v)
			{
				ASSERT_FALSE(vz::compare_exact(dst, rhs));
			}

			cv::imwrite(name + "_gaussian_blur_" + std::to_string(k) + "x" + std::to_string(k) + ".jpg", dst, {cv::IMWRITE_JPEG_QUALITY, 75, cv::IMWRITE_JPEG_OPTIMIZE, 1});

			// (except for sudoku, which is so thin that at the upper values of "k", the images look the same
			if (name != "sudoku" && name != "sudoku_312")
			{
				v.push_back(dst);
			}
		}

		// the source image should not have changed
		ASSERT_TRUE(vz::compare_exact(src, copy));
	}
}
